@@ -30,11 +30,10 @@ logger = logging.getLogger(__name__)
 
 
 class MongoDbUtil(object):
-
     def __init__(self, mongoClientObj, verbose=False):
         self.__verbose = verbose
         self.__mgObj = mongoClientObj
-        self.__mongoIndexTypes = {'DESCENDING': pymongo.DESCENDING, 'ASCENDING': pymongo.ASCENDING, 'TEXT': pymongo.TEXT}
+        self.__mongoIndexTypes = {"DESCENDING": pymongo.DESCENDING, "ASCENDING": pymongo.ASCENDING, "TEXT": pymongo.TEXT}
 
     def databaseExists(self, databaseName):
         try:
@@ -44,7 +43,7 @@ class MongoDbUtil(object):
             else:
                 return False
         except Exception as e:
-            logger.exception("Failing with %s" % str(e))
+            logger.exception("Failing with %s", str(e))
         return False
 
     def getDatabaseNames(self):
@@ -53,13 +52,13 @@ class MongoDbUtil(object):
     def createDatabase(self, databaseName, overWrite=True):
         try:
             if overWrite and self.databaseExists(databaseName):
-                logger.debug("Dropping existing database %s" % databaseName)
+                logger.debug("Dropping existing database %s", databaseName)
                 self.__mgObj.drop_database(databaseName)
             db = self.__mgObj[databaseName]
-            logger.debug("Creating database %s %r" % (databaseName, db))
+            logger.debug("Creating database %s %r", databaseName, db)
             return True
         except Exception as e:
-            logger.exception("Failing with %s" % str(e))
+            logger.exception("Failing with %s", str(e))
 
         return False
 
@@ -68,16 +67,16 @@ class MongoDbUtil(object):
             self.__mgObj.drop_database(databaseName)
             return True
         except Exception as e:
-            logger.exception("Failing with %s" % str(e))
+            logger.exception("Failing with %s", str(e))
         return False
 
     def databaseCommand(self, databaseName, command):
         try:
             rs = self.__mgObj[databaseName].command(command)
-            logger.debug("Database %s Command %r returns %r" % (databaseName, command, rs))
+            logger.debug("Database %s Command %r returns %r", databaseName, command, rs)
             return True
         except Exception as e:
-            logger.exception("Database %s command %s failing with %s" % (databaseName, command, str(e)))
+            logger.exception("Database %s command %s failing with %s", databaseName, command, str(e))
         return False
 
     def collectionExists(self, databaseName, collectionName):
@@ -87,13 +86,13 @@ class MongoDbUtil(object):
             else:
                 return False
         except Exception as e:
-            logger.exception("Failing with %s" % str(e))
+            logger.exception("Failing with %s", str(e))
         return False
 
     def getCollectionNames(self, databaseName):
         return self.__mgObj[databaseName].list_collection_names()
 
-    def createCollection(self, databaseName, collectionName, overWrite=True, bsonSchema=None, validationLevel='strict', validationAction='error'):
+    def createCollection(self, databaseName, collectionName, overWrite=True, bsonSchema=None, validationLevel="strict", validationAction="error"):
         """
         Args:
             databaseName (str): Description
@@ -113,43 +112,41 @@ class MongoDbUtil(object):
                 self.__mgObj[databaseName].drop_collection(collectionName)
             #
             ok = self.__mgObj[databaseName].create_collection(collectionName)
-            logger.debug("Return from create collection %r " % ok)
+            logger.debug("Return from create collection %r", ok)
             if bsonSchema:
+                # bsonSchema.update({'additionalProperties': False})
                 sD = {"$jsonSchema": bsonSchema}
-                cmdD = OrderedDict([('collMod', collectionName),
-                                    ('validator', sD),
-                                    ('validationLevel', validationLevel),
-                                    ('validationAction', validationAction)])
+                cmdD = OrderedDict([("collMod", collectionName), ("validator", sD), ("validationLevel", validationLevel), ("validationAction", validationAction)])
                 self.__mgObj[databaseName].command(cmdD)
             return True
         except Exception as e:
-            logger.exception("Failing for databaseName %s collectionName %s with %s" % (databaseName, collectionName, str(e)))
+            logger.exception("Failing for databaseName %s collectionName %s with %s", databaseName, collectionName, str(e))
         return False
 
     def dropCollection(self, databaseName, collectionName):
         try:
             ok = self.__mgObj[databaseName].drop_collection(collectionName)
-            logger.debug("Return from drop collection %r " % ok)
+            logger.debug("Return from drop collection %r", ok)
             return True
         except Exception as e:
-            logger.error("Failing drop collection for databaseName %s collectionName %s with %s" % (databaseName, collectionName, str(e)))
+            logger.error("Failing drop collection for databaseName %s collectionName %s with %s", databaseName, collectionName, str(e))
         return False
 
     def insert(self, databaseName, collectionName, dObj, documentKey=None):
         try:
-            c = self.__mgObj[databaseName].get_collection(collectionName)
-            r = c.insert_one(dObj)
+            clt = self.__mgObj[databaseName].get_collection(collectionName)
+            rV = clt.insert_one(dObj)
             try:
-                rId = r.inserted_id
+                rId = rV.inserted_id
                 return rId
             except Exception as e:
-                logger.debug("Failing with %s" % str(e))
+                logger.debug("Failing with %s", str(e))
                 return None
         except Exception as e:
             if documentKey:
-                logger.error("Failing %r with %s" % (documentKey, str(e)))
+                logger.error("Failing %r with %s", documentKey, str(e))
             else:
-                logger.error("Failing with %s" % str(e))
+                logger.error("Failing with %s", str(e))
         return None
 
     def insertList(self, databaseName, collectionName, dList, ordered=False, bypassValidation=False, keyNames=None, salvage=False):
@@ -171,61 +168,24 @@ class MongoDbUtil(object):
 
         """
         rIdL = []
-        r = None
+        rV = None
         try:
-            c = self.__mgObj[databaseName].get_collection(collectionName)
-            r = c.insert_many(dList, ordered=ordered, bypass_document_validation=bypassValidation)
+            clt = self.__mgObj[databaseName].get_collection(collectionName)
+            rV = clt.insert_many(dList, ordered=ordered, bypass_document_validation=bypassValidation)
         except Exception as e:
-            logger.error("Bulk insert failing for document length %d with %s" % (len(dList), str(e)))
+            logger.error("Bulk insert failing for document length %d with %s", len(dList), str(e))
+            # for ii, dd in enumerate(dList):
+            #    logger.error(" %d error doc %r" % (ii, list(dd.keys())))
         #
         try:
-            rIdL = r.inserted_ids if r is not None else []
+            rIdL = rV.inserted_ids if rV is not None else []
         except Exception as e:
-            logger.error("Bulk insert list processing fails for document length %d with %s" % (len(dList), str(e)))
+            logger.error("Bulk insert list processing fails for document length %d with %s", len(dList), str(e))
 
         if salvage and keyNames and (len(rIdL) < len(dList)):
-            logger.info("Bulk insert document recovery starting for %d documents " % len(dList))
+            logger.info("Bulk insert document recovery starting for %d documents", len(dList))
             rIdL = self.__salvageinsertList(databaseName, collectionName, dList, keyNames)
-            logger.info("Bulk insert document recovery returns %d of %d" % (len(rIdL), len(dList)))
-
-        return rIdL
-
-    def XinsertList(self, databaseName, collectionName, dList, ordered=False, bypassValidation=False, keyNames=None, salvage=False):
-        """Insert the input list of documents (dList) into the input database/collection.
-
-
-        Args:
-            databaseName (str): Target database name
-            collectionName (str): Target collection name
-            dList (list): document list
-            ordered (bool, optional): insert in input order
-            bypassValidation (bool, optional): skip internal validation processing
-            keyNames (list, optional): list of key names required to uniquely identify the object (dot notation)
-            salvage (bool, optional): perform serial salvage operation for a batch insert failure
-
-        Returns:
-            list: List of MongoDB document identifiers for inserted objects
-
-
-        """
-        rIdL = []
-        r = None
-        try:
-            c = self.__mgObj[databaseName].get_collection(collectionName)
-            r = c.insert_many(dList, ordered=ordered, bypass_document_validation=bypassValidation)
-        except Exception as e:
-            logger.error("Bulk insert failing for document length %d with %s" % (len(dList), str(e)))
-        #
-        try:
-            rIdL = r.inserted_ids if r is not None else []
-            return rIdL
-        except Exception as e:
-            rIdL = []
-            logger.info("Bulk insert document recovery starting for %d documents with %s" % (len(dList), str(e)))
-            if salvage and keyNames:
-                sIdL = self.__salvageinsertList(databaseName, collectionName, dList, keyNames)
-            logger.info("Bulk insert document recovery returns %d of %d" % (len(sIdL), len(dList)))
-            return sIdL
+            logger.info("Bulk insert document recovery returns %d of %d", len(rIdL), len(dList))
 
         return rIdL
 
@@ -244,16 +204,16 @@ class MongoDbUtil(object):
         """
         rIdL = []
         try:
-            for d in dList:
-                kyVals = self.__getKeyValues(d, keyNames)
-                rId = self.insert(databaseName, collectionName, d, documentKey=kyVals)
+            for dD in dList:
+                kyVals = self.__getKeyValues(dD, keyNames)
+                rId = self.insert(databaseName, collectionName, dD, documentKey=kyVals)
                 if rId:
                     rIdL.append(rId)
-                    logger.debug("Insert suceeds for document %s" % (kyVals,))
+                    logger.debug("Insert suceeds for document %s", (kyVals,))
                 else:
-                    logger.debug("Loading document %r failed" % (kyVals,))
+                    logger.debug("Loading document %r failed", (kyVals,))
         except Exception as e:
-            logger.exception("Failing %s and %s keyName %r with %s" % (databaseName, collectionName, keyNames, str(e)))
+            logger.exception("Failing %s and %s keyName %r with %s", databaseName, collectionName, keyNames, str(e))
         #
         return rIdL
 
@@ -270,22 +230,52 @@ class MongoDbUtil(object):
             list: List of MongoDB document identifiers for inserted objects
 
         """
-        logger.info("Salvaging %s %s document list length %d" % (databaseName, collectionName, len(dList)))
+        logger.info("Salvaging %s %s document list length %d", databaseName, collectionName, len(dList))
         dTupL = self.deleteList(databaseName, collectionName, dList, keyNames)
-        logger.info("Salvage bulk insert - deleting %d documents" % len(dTupL))
+        logger.info("Salvage bulk insert - deleting %d documents", len(dTupL))
         rIdL = self.insertListSerial(databaseName, collectionName, dList, keyNames)
-        logger.info("Salvage bulk insert - salvaged document length %d" % len(rIdL))
+        logger.info("Salvage bulk insert - salvaged document length %d", len(rIdL))
         return rIdL
 
     def fetchOne(self, databaseName, collectionName, ky, val):
         try:
-            c = self.__mgObj[databaseName].get_collection(collectionName)
-            dObj = c.find_one({ky: val})
+            clt = self.__mgObj[databaseName].get_collection(collectionName)
+            dObj = clt.find_one({ky: val})
             return dObj
         except Exception as e:
-            logger.exception("Failing with %s" % str(e))
+            logger.exception("Failing with %s", str(e))
         return None
 
+    def update(self, databaseName, collectionName, dObj, selectD):
+        """ Update documents satisfying the selection details with the content of dObj.
+
+        Args:
+            databaseName (str): Target database name
+            collectionName (str): Target collection name
+            dObj (dict): document data (dotted notation for sub-objects applied with '$set')
+            selectD (dict): dictionary of key/values for the selction/filter query
+
+        Returns:
+            int: update document count
+
+            update_many(filter, update, upsert=False, array_filters=None)
+        """
+        try:
+            numModified = 0
+            clt = self.__mgObj[databaseName].get_collection(collectionName)
+            rV = clt.update_many(selectD, {"$set": dObj}, upsert=False, array_filters=None)
+            try:
+                numMatched = rV.matched_count
+                numModified = rV.modified_count
+                logger.debug("Replacement matched %d modified %d", numMatched, numModified)
+                return numModified
+            except Exception as e:
+                logger.error("Failing update %s and %s selectD %r with %s", databaseName, collectionName, selectD, str(e))
+                return None
+        except Exception as e:
+            logger.exception("Failing update %s and %s selectD %r with %s", databaseName, collectionName, selectD, str(e))
+        return numModified
+
     def replace(self, databaseName, collectionName, dObj, selectD, upsertFlag=True):
         """Replace the input document based on a selection query in the input selection dictionary (k,v).
 
@@ -293,7 +283,7 @@ class MongoDbUtil(object):
             databaseName (str): Target database name
             collectionName (str): Target collection name
             dList (list): document list
-            selectD (dict, optional): dictiony of key/values for the selction query
+            selectD (dict, optional): dictionary of key/values for the selction query
             upsertFlag (bool, optional): set MongoDB 'upsert' option
 
         Returns:
@@ -301,20 +291,20 @@ class MongoDbUtil(object):
 
         """
         try:
-            c = self.__mgObj[databaseName].get_collection(collectionName)
-            r = c.replace_one(selectD, dObj, upsert=upsertFlag)
-            logger.debug("Replace returns  %r" % r)
+            clt = self.__mgObj[databaseName].get_collection(collectionName)
+            rV = clt.replace_one(selectD, dObj, upsert=upsertFlag)
+            logger.debug("Replace returns  %r", rV)
             try:
-                rId = r.upserted_id
-                numMatched = r.matched_count
-                numModified = r.modified_count
-                logger.debug("Replacement mathed %d modified %d with _id %s" % (numMatched, numModified, rId))
-                return rId
+                rId = rV.upserted_id
+                numMatched = rV.matched_count
+                numModified = rV.modified_count
+                logger.debug("Replacement matched %d modified %d or upserted with _id %s", numMatched, numModified, rId)
+                return numMatched or numModified
             except Exception as e:
-                logger.error("Failing %s and %s selectD %r with %s" % (databaseName, collectionName, selectD, str(e)))
+                logger.error("Failing %s and %s selectD %r with %s", databaseName, collectionName, selectD, str(e))
                 return None
         except Exception as e:
-            logger.exception("Failing %s and %s selectD %r with %s" % (databaseName, collectionName, selectD, str(e)))
+            logger.exception("Failing %s and %s selectD %r with %s", databaseName, collectionName, selectD, str(e))
         return None
 
     def replaceList(self, databaseName, collectionName, dList, keyNames, upsertFlag=True):
@@ -333,17 +323,17 @@ class MongoDbUtil(object):
         """
         try:
             rIdL = []
-            c = self.__mgObj[databaseName].get_collection(collectionName)
-            for d in dList:
-                kyVals = self.__getKeyValues(d, keyNames)
+            clt = self.__mgObj[databaseName].get_collection(collectionName)
+            for dD in dList:
+                kyVals = self.__getKeyValues(dD, keyNames)
                 selectD = {ky: val for ky, val in zip(keyNames, kyVals)}
-                r = c.replace_one(selectD, d, upsert=upsertFlag)
+                rV = clt.replace_one(selectD, dD, upsert=upsertFlag)
                 try:
-                    rIdL.append(r.upserted_id)
+                    rIdL.append(rV.upserted_id)
                 except Exception as e:
-                    logger.error("Failing for %s and %s selectD %r with %s" % (databaseName, collectionName, selectD.items(), str(e)))
+                    logger.error("Failing for %s and %s selectD %r with %s", databaseName, collectionName, selectD.items(), str(e))
         except Exception as e:
-            logger.error("Failing %s and %s selectD %r with %s" % (databaseName, collectionName, selectD.items(), str(e)))
+            logger.error("Failing %s and %s selectD %r with %s", databaseName, collectionName, selectD.items(), str(e))
         #
         return rIdL
 
@@ -364,24 +354,24 @@ class MongoDbUtil(object):
         try:
             cD = {}
             delTupL = []
-            c = self.__mgObj[databaseName].get_collection(collectionName)
-            for d in dList:
-                kyVals = self.__getKeyValues(d, keyNames)
+            clt = self.__mgObj[databaseName].get_collection(collectionName)
+            for dD in dList:
+                kyVals = self.__getKeyValues(dD, keyNames)
                 selectD = {ky: val for ky, val in zip(keyNames, kyVals)}
                 tt = tuple(selectD.items())
                 if tt in cD:
                     continue
                 cD[tt] = True
-                r = c.delete_many(selectD)
+                rV = clt.delete_many(selectD)
                 try:
                     # delTupL.append((kyVals, r.deleted_count))
-                    delTupL.append((selectD, r.deleted_count))
+                    delTupL.append((selectD, rV.deleted_count))
                 except Exception as e:
-                    logger.error("Failing %s and %s selectD %r with %s" % (databaseName, collectionName, selectD.items(), str(e)))
-            logger.debug("%s %s deleted status %r" % (databaseName, collectionName, delTupL))
+                    logger.error("Failing %s and %s selectD %r with %s", databaseName, collectionName, selectD.items(), str(e))
+            logger.debug("%s %s deleted status %r", databaseName, collectionName, delTupL)
             return delTupL
         except Exception as e:
-            logger.error("Failing %s and %s selectD %r with %s" % (databaseName, collectionName, selectD.items(), str(e)))
+            logger.error("Failing %s and %s selectD %r with %s", databaseName, collectionName, selectD.items(), str(e))
         #
         return delTupL
 
@@ -389,35 +379,35 @@ class MongoDbUtil(object):
 
         try:
             iTupL = [(ky, self.__mongoIndexTypes[indexType]) for ky in keyList]
-            c = self.__mgObj[databaseName].get_collection(collectionName)
-            c.create_index(iTupL, name=indexName, background=True, unique=uniqueFlag)
-            logger.debug("Current indexes for %s %s : %r" % (databaseName, collectionName, c.list_indexes()))
+            clt = self.__mgObj[databaseName].get_collection(collectionName)
+            clt.create_index(iTupL, name=indexName, background=True, unique=uniqueFlag)
+            logger.debug("Current indexes for %s %s : %r", databaseName, collectionName, clt.list_indexes())
             return True
         except Exception as e:
-            logger.error("Failing %s and %s keyList %r with %s" % (databaseName, collectionName, keyList, str(e)))
+            logger.error("Failing %s and %s keyList %r with %s", databaseName, collectionName, keyList, str(e))
         return False
 
     def dropIndex(self, databaseName, collectionName, indexName="primary"):
         try:
-            c = self.__mgObj[databaseName].get_collection(collectionName)
-            c.drop_index(indexName)
-            logger.debug("Current indexes for %s %s : %r" % (databaseName, collectionName, c.list_indexes()))
+            clt = self.__mgObj[databaseName].get_collection(collectionName)
+            clt.drop_index(indexName)
+            logger.debug("Current indexes for %s %s : %r", databaseName, collectionName, clt.list_indexes())
             return True
         except Exception as e:
-            logger.error("Failing %s and %s with %s" % (databaseName, collectionName, str(e)))
+            logger.error("Failing %s and %s with %s", databaseName, collectionName, str(e))
         return False
 
     def reIndex(self, databaseName, collectionName):
         try:
-            c = self.__mgObj[databaseName].get_collection(collectionName)
-            logger.debug("Current indexes for %s %s : %r" % (databaseName, collectionName, c.list_indexes()))
-            c.reindex()
+            clt = self.__mgObj[databaseName].get_collection(collectionName)
+            logger.debug("Current indexes for %s %s : %r", databaseName, collectionName, clt.list_indexes())
+            clt.reindex()
             return True
         except Exception as e:
-            logger.exception("Failing %s and %s with %s" % (databaseName, collectionName, str(e)))
+            logger.exception("Failing %s and %s with %s", databaseName, collectionName, str(e))
         return False
 
-    def fetch(self, databaseName, collectionName, selectL, queryD=None):
+    def fetch(self, databaseName, collectionName, selectL, queryD=None, suppressId=False):
         """ Fetch selections (selectL) from documents satisfying input
             query constraints.
         """
@@ -426,25 +416,31 @@ class MongoDbUtil(object):
             qD = queryD if queryD is not None else None
             if selectL:
                 sD = {k: 1 for k in selectL}
+                if suppressId:
+                    sD["_id"] = 0
             else:
-                sD = None
-            c = self.__mgObj[databaseName].get_collection(collectionName)
+                if suppressId:
+                    sD["_id"] = 0
+                else:
+                    sD = None
+            clt = self.__mgObj[databaseName].get_collection(collectionName)
             # logger.debug("Got collection object %r" % c)
-            for d in c.find(filter=qD, projection=sD):
+            for dD in clt.find(filter=qD, projection=sD):
                 # logger.debug("Got doc %r" % d)
-                dList.append(d)
+                dList.append(dD)
             return dList
         except Exception as e:
-            logger.exception("Failing with %s" % str(e))
+            logger.exception("Failing with %s", str(e))
         return None
 
-    def count(self, databaseName, collectionName, filter={}):
+    def count(self, databaseName, collectionName, countFilter=None):
         try:
-            c = self.__mgObj[databaseName].get_collection(collectionName)
-            logger.debug("Current indexes for %s %s : %r" % (databaseName, collectionName, c.list_indexes()))
-            return c.count_documents(filter)
+            tF = countFilter if countFilter else {}
+            clt = self.__mgObj[databaseName].get_collection(collectionName)
+            logger.debug("Current indexes for %s %s : %r", databaseName, collectionName, clt.list_indexes())
+            return clt.count_documents(tF)
         except Exception as e:
-            logger.exception("Failing %s and %s with %s" % (databaseName, collectionName, str(e)))
+            logger.exception("Failing %s and %s with %s", databaseName, collectionName, str(e))
         return 0
 
     def __getKeyValues(self, dct, keyNames):
@@ -463,7 +459,7 @@ class MongoDbUtil(object):
             for keyName in keyNames:
                 rL.append(self.__getKeyValue(dct, keyName))
         except Exception as e:
-            logger.exception("Failing for key names %r with %s" % (keyNames, str(e)))
+            logger.exception("Failing for key names %r with %s", keyNames, str(e))
 
         return tuple(rL)
 
@@ -471,7 +467,7 @@ class MongoDbUtil(object):
         """  Return the value of the corresponding key expressed in dot notation in the input dictionary object (nested).
         """
         try:
-            kys = keyName.split('.')
+            kys = keyName.split(".")
             for key in kys:
                 try:
                     dct = dct[key]
@@ -479,6 +475,6 @@ class MongoDbUtil(object):
                     return None
             return dct
         except Exception as e:
-            logger.exception("Failing for key %r with %s" % (keyName, str(e)))
+            logger.exception("Failing for key %r with %s", keyName, str(e))
 
         return None
